#include "gloobus-preview-singleton.h"



//// ============================= SINGLETON =========================== //
Gloobus * Gloobus::m_instance = 0;
Gloobus * Gloobus::instance()
{ 
  	if (m_instance == NULL)
  	{
  		m_instance = new Gloobus();
  	}
  	return m_instance; 
}

Gloobus::Gloobus() 
{ 
    //... Contructor
}

//// ============================== INIT ================================ //
void Gloobus::init(string filename ,gloobusWin *win)
{	
	m_filename 		= filename;
	m_current_file 	= 0;
	m_gwindow		= win;
	
	m_PManager.load_plugins	();			//Plugin manager loads the .so
	load_folder_filenames	();			//It stores the filesnames in the folder
	search_actual_filename	();			//It looks if wich is the file we are showing
}
// =============================== UNLOAD PLUGIN ======================= //
// Unload all the plugins
void Gloobus::end()
{
	m_PManager.unload_plugins();						// Unload Plugins
}
// =============================== CREATE PLUGIN ======================= //
// This functions searchs the plugin associated with the <m_filename> and then creates it.
// It also defines the variable <m_gloobus_interface> to be a polymorphism so the default functions
// can be called with directly on this variable
void Gloobus::create_plugin( )
{
	m_plugin_name = "";
	m_class		  = "";
	
	m_plugin_name 	 = m_PManager.search_for_plugin(m_filename);
	m_class 		 = m_PManager.get_classtype ();
	g_print("Creating Plugin Class: %s \n",m_class.c_str());
	bool plugin_found = false;									//If there is no plugin, we use default one
	
	if(m_class == "image")
	{
		plugin_found =true;
		g_print("IMAGE (%s)\n",m_plugin_name.c_str());
		m_image = factory_image[m_plugin_name.c_str()]();
		m_gloobus_interface = m_image;							//Polymorphism
	}
	if(m_class == "music")
	{
		plugin_found =true;
		g_print("MUSIC (%s)\n",m_plugin_name.c_str());
		m_music = factory_music[m_plugin_name.c_str()]();
		m_gloobus_interface = m_music;							//Polymorphism
	}
	if(m_class == "text")
	{
		plugin_found =true;
		g_print("TEXT (%s)\n",m_plugin_name.c_str());
		m_text = factory_text[m_plugin_name.c_str()]();
		m_gloobus_interface = m_text;

	}
	if(m_class == "document")
	{
		plugin_found =true;
		g_print("DOCUMENT (%s)\n",m_plugin_name.c_str());
		m_document = factory_document[m_plugin_name.c_str()]();
		m_gloobus_interface = m_document;						//Polymorphism
	}
	if(m_class == "list")
	{
		plugin_found =true;
		g_print("LIST (%s)\n",m_plugin_name.c_str());
		m_list = factory_list[m_plugin_name.c_str()]();
		m_gloobus_interface = m_list;							//Polymorphism
	}
	if(m_class == "movie")
	{
		plugin_found =true;
		g_print("MOVIE (%s)\n",m_plugin_name.c_str());
		m_movie = factory_movie[m_plugin_name.c_str()]();
		m_gloobus_interface = m_movie;							//Polymorphism
	}
	if(m_class == "free")
	{
		plugin_found =true;
		g_print("FREE (%s)\n",m_plugin_name.c_str());
		m_free = factory_free[m_plugin_name.c_str()]();
		m_gloobus_interface = m_free;							//Polymorphism
	}
	if(!plugin_found)	
	{
		g_print("DEFAULT\n");
		m_class = "default";
		m_default = new iDefault();
		m_gloobus_interface = m_default;						//Polymorphism
	}
	
	m_gloobus_interface->set_filename(m_filename);
	m_gloobus_interface->load();
}
// ============================= DESTROY PLUGIN ======================= //
void Gloobus::destroy_plugin()
{
	g_print("Destory Plugin:\n");
	m_gloobus_interface->end();				//Call a ending function before the destructor
											//If we don't do this, thread functions calling virtual plugin function may crash
	
	if(m_class == "image") 		{ factoryDestroy_image		[m_plugin_name.c_str()]	(m_image); 		}
	if(m_class == "music") 		{ m_music->stop();	factoryDestroy_music [m_plugin_name.c_str()]	(m_music); 	}
	if(m_class == "movie") 		{ m_movie->stop();	factoryDestroy_movie [m_plugin_name.c_str()]	(m_movie); 	}
	if(m_class == "text")  		{ factoryDestroy_text		[m_plugin_name.c_str()]	(m_text); 		}
	//if(m_class == "document")	{ factoryDestroy_document	[m_plugin_name.c_str()]	(m_document); 	}	//It crashed when changing 		from iDocument
	if(m_class == "list")		{ factoryDestroy_list		[m_plugin_name.c_str()]	(m_list); 		}
	if(m_class == "free")		{ factoryDestroy_free		[m_plugin_name.c_str()]	(m_free); 		}
	if(m_class == "default")	{ delete m_default; }
	

}
// ============================= GET WIDTH ======================= //
int Gloobus::get_width(void)
{

	return m_gloobus_interface->get_width();

}
// ============================= GET HEIGHT ======================= //
int Gloobus::get_height(void)
{
	return m_gloobus_interface->get_height();

}
// ============================= DRAW PLUGIN ======================= //
void Gloobus::draw_plugin(void)
{
	m_gloobus_interface->draw(GTK_CONTAINER(m_gwindow->get_container_pointer()));
}
// ============================= RESIZE WINDOW ======================= //
void Gloobus::resize_window	( int x, int y )
{
	gtk_window_resize(GTK_WINDOW(m_gwindow->get_window_pointer()),x,y);
}
// ============================= FULLSCREEN WINDOW ======================= //
void Gloobus::fullscreen (void)
{
	GdkScreen *screen = gtk_window_get_screen(GTK_WINDOW(m_gwindow->get_window_pointer()));

	if(m_fullscreen)
	{
		gtk_window_unfullscreen(GTK_WINDOW(m_gwindow->get_window_pointer()));
		m_gwindow->set_size(get_width()+SHADOW_WIDTH*2 ,get_height()+SHADOW_WIDTH*2+HEADER_HEIGHT+BOTTOM_HEIGHT);
		m_fullscreen = false;
	}
	else
	{
		gtk_window_fullscreen(GTK_WINDOW(m_gwindow->get_window_pointer()));
		m_gwindow->set_size(gdk_screen_get_width(screen),gdk_screen_get_height(screen));
		m_fullscreen = true;
	}
	

}
// ============================= Load filenames ======================= //
void Gloobus::load_folder_filenames  ( void )
{

	string filename = m_filename.substr(0,m_filename.rfind("/"));
	string folder =   m_filename.substr(0,m_filename.rfind("/"));
	g_print("CONTENT FOLDER: %s\n", folder.c_str());
	
    DIR *dp;
    struct dirent *dirp;
    if((dp  = opendir(folder.c_str())) == NULL) {
        cout << "Error(" << errno << ") opening " << folder << endl;
    }

    while ((dirp = readdir(dp)) != NULL) 
    {
		if(string(dirp->d_name) != "." && string(dirp->d_name) != "..")  
		{
        	m_files.push_back (string(folder + "/" + dirp->d_name));
        	//g_print("File: %s\n", m_files[m_files.size()-1].c_str());
		}
    }
    
    std::sort( m_files.begin(), m_files.end() );
    closedir(dp);

}
// ============================= Search actual filename ======================= //
void Gloobus::search_actual_filename ( void )
{
	for(unsigned int i=0; i< m_files.size()-1; i++)
	{
		//g_print("%s - %s\n", m_files[i].c_str(), m_filename.c_str());
		if(m_files[i] == m_filename)
		{
			 m_current_file = i;
			 g_print("Actual file number: %d\n",i);
		}
	}
}
// ============================= Load Next File ======================= //
void Gloobus::next_file ( void )
{
	m_current_file++;
	if((unsigned int)m_current_file == m_files.size()) m_current_file = 0;
	
	g_print("\n\nLoading next file (%i): %s\n", m_current_file, m_files[m_current_file].c_str());
	m_filename = m_files[m_current_file];

	end_file_and_new_file();

}
// =============================== Load prev File ========================= //
void Gloobus::prev_file ( void )
{
	m_current_file--;
	if(m_current_file < 0) m_current_file = m_files.size() - 1;
	
	g_print("\n\nLoading previous file (%i): %s\n", m_current_file, m_files[m_current_file].c_str());
	m_filename = m_files[m_current_file];

	
	end_file_and_new_file();

}
// ============================= END File and new file ======================= //
// This functions clears the current window and prepare it for the new file
// after clearing the window, it draws the new file
void Gloobus::end_file_and_new_file ( void )
{

	destroy_plugin();
	gtk_widget_destroy	(GTK_WIDGET(m_gwindow->get_container_pointer()));

	m_gwindow->create_container();							//Create a new container and

	create_plugin();
	
	m_gwindow->set_size(get_width() +  SHADOW_WIDTH*2, get_height()+ SHADOW_WIDTH*2 + BOTTOM_HEIGHT+HEADER_HEIGHT);	
	m_gwindow->set_title		  (m_filename);
	m_gwindow->add_common_buttons ();
	m_gwindow->show_all			  ();

	draw_plugin();


}
// ============================= Launch default application ======================= //
// Launch the gnome-open function for the current file
void Gloobus::launch_default_application ( void )
{
	string cmd = "gnome-open \"" +  m_filename + "\"" ; 
	g_spawn_command_line_async(cmd.c_str(), NULL);
}

