#include "plugin-gstreamer-video.h"

iMpg::iMpg(){}
iMpg::~iMpg()
{
	g_print("PLUGIN: iMpg Destroyed\n");
	gst_object_unref (GST_OBJECT (m_pipeline));
}


// ================================ LOAD ============================== //
void iMpg::play()
{
	g_print("Start Playing (%s)...\n",g_file_get_uri ( m_gfile));

	gst_init (NULL, NULL);

	m_pipeline	= gst_pipeline_new 		   ("gst-player");
	m_bin 		= gst_element_factory_make ("playbin", 		"m_bin");
	m_sink 		= gst_element_factory_make ("xvimagesink", 	"m_sink");
	g_object_set (G_OBJECT (m_bin), "video-sink", m_sink, NULL);
	gst_bin_add  (GST_BIN (m_pipeline), m_bin);


	{
		GstBus *bus;
		bus = gst_pipeline_get_bus (GST_PIPELINE (m_pipeline));
		gst_object_unref (bus);
	}


	g_object_set (G_OBJECT (m_bin), "uri", g_file_get_uri( m_gfile), NULL);
	g_object_set (G_OBJECT (m_sink), "force-aspect-ratio", TRUE, NULL);
	
    	
    if (GST_IS_X_OVERLAY (m_sink))
    {
		//printf("Is Overlay (Win ID: %i)!!\n",GPOINTER_TO_INT (m_window));
	
		gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (m_sink),GDK_WINDOW_XWINDOW (m_drawable->window));
		gst_x_overlay_handle_events  (GST_X_OVERLAY (m_sink), FALSE);
    }
 
	gst_element_set_state (m_pipeline, GST_STATE_PLAYING);

}
// =============================== STOP ============================= //
void iMpg::stop()
{
	gst_element_set_state (GST_ELEMENT (m_pipeline), GST_STATE_READY);

}

// ================================== SEEK =================================== //
// Move the pipeline to the value (value range is from 0 to 100) so
// a division with the duration must be done
void iMpg::seek	( double value )
{
	//if(!DURATION_IS_VALID (m_duration))
		m_duration = get_duration();
		
	 if (DURATION_IS_VALID (m_duration))
	 {	 
		 
		GstSeekFlags seek_flags = GstSeekFlags(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT);
		guint64 to_seek = m_duration  * value / 100;
		//guint64 to_seek = 13899408005078;
		//g_print("PLUGIN: SEEK to %d (%d)\n",value,to_seek);
		//g_print("PLUGIN: SEEK to %.2f -Seek: %llu - Duration: %llu\n",value,to_seek,m_duration);

		gst_element_seek (m_pipeline, 1.0,
				          GST_FORMAT_TIME,
				          GST_SEEK_FLAG_FLUSH,
				          GST_SEEK_TYPE_SET, to_seek,
				          GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
    }
    else
    {
    	g_print("Invalid Duration\n");
    }
}


// =============================== GET DURATION ============================= //
guint64 iMpg::get_duration( void )
{

    GstFormat format = GST_FORMAT_TIME;
    gint64 cur;
    gboolean result;

    result = gst_element_query_duration (m_pipeline, &format, &cur);
    if (!result || format != GST_FORMAT_TIME)
        return GST_CLOCK_TIME_NONE;

    return cur;
}
// =============================== GET DURATION ============================= //
double iMpg::get_position	( void )
{
	m_duration = get_duration();
    GstFormat format = GST_FORMAT_TIME;
    gint64 cur;
    gboolean result;

    result = gst_element_query_position (m_pipeline, &format, &cur);
    if (!result || format != GST_FORMAT_TIME)
        return 0;

    return  cur * (((double) 100) / m_duration);

}
